package coco;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTVisitor;
import org.eclipse.jdt.core.dom.AnnotationTypeDeclaration;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.EnumDeclaration;
import org.eclipse.jdt.core.dom.TypeDeclaration;
import org.eclipse.jdt.core.dom.rewrite.ASTRewrite;

/**
 * Top {@link ASTVisitor} for CoCo plugin.
 * 
 * @author Milos Gligoric (milos.gligoric@gmail.com)
 * @author Rohan Sharma (sharma.rohan1990@gmail.com)
 */
public class CoCoASTV extends ASTVisitor implements ICoCoASTV {

    /** AST of the CU being visited */
    private AST ast;

    /** ASTRewrite for the CU being visited */
    private ASTRewrite rewrite;

    protected final AST ast() {
        return ast;
    }

    @Override
    public final ASTRewrite rewrite() {
        return rewrite;
    }

    // ASTVisitor interface

    @Override
    public final boolean visit(CompilationUnit node) {
        ast = node.getAST();
        rewrite = ASTRewrite.create(node.getAST());
        return oVisit(node);
    }

    protected boolean oVisit(CompilationUnit node) {
        return true;
    }

    @Override
    public final boolean visit(TypeDeclaration node) {
        if (node.isInterface()) {
            // if (Flags.isAbstract(node.getFlags())
            // || Modifier.isAbstract(node.getModifiers())) {
            return false;
        } else {
            return oVisit(node);
        }
    }

    protected boolean oVisit(TypeDeclaration node) {
        // see subclasses
        return true;
    }

    @Override
    public final void endVisit(TypeDeclaration node) {
        if (!node.isInterface()) {
            // if (!(Flags.isAbstract(node.getFlags()) ||
            // Modifier
            // .isAbstract(node.getModifiers()))) {
            oEndVisit(node);
        }
    }

    protected void oEndVisit(TypeDeclaration node) {
        // see subclasses
    }

    @Override
    public final boolean visit(AnnotationTypeDeclaration node) {
        // ignoring annotation type declarations
        return false;
    }

    @Override
    public final void endVisit(AnnotationTypeDeclaration node) {
        // ignoring annotation type declarations
    }

    @Override
    public final boolean visit(EnumDeclaration node) {
        // ignoring enum declarations
        return false;
    }

    @Override
    public final void endVisit(EnumDeclaration node) {
        // ignoring enum declarations
    }
}
