package coco.task;

import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.ltk.core.refactoring.Change;
import org.eclipse.ltk.core.refactoring.CheckConditionsOperation;
import org.eclipse.ltk.core.refactoring.CreateChangeOperation;
import org.eclipse.ltk.core.refactoring.PerformChangeOperation;
import org.eclipse.ltk.core.refactoring.Refactoring;
import org.eclipse.ltk.core.refactoring.RefactoringStatus;

/**
 * Util class for performing refactorings.
 * 
 * @author Milos Gligoric (milos.gligoric@gmail.com)
 * @author Rohan Sharma (sharma.rohan1990@gmail.com)
 */
public final class PerformRefactoring {

    /** Only the latest change is saved */
    private static Change undoChange;

    public static ProblemReport performRefactoring(Refactoring refactoring) throws CoreException {
        return performRefactoring(refactoring, false);
    }

    public static ProblemReport performRefactoring(Refactoring refactoring, boolean saveUndo) throws CoreException {
        CreateChangeOperation create = new CreateChangeOperation(new CheckConditionsOperation(refactoring,
                CheckConditionsOperation.FINAL_CONDITIONS), RefactoringStatus.FATAL);
        PerformChangeOperation perform = new PerformChangeOperation(create);

        IWorkspace workspace = ResourcesPlugin.getWorkspace();
        workspace.run(perform, EmptyProgressMonitor.inst());
        if (saveUndo) {
            undoChange = perform.getUndoChange();
        }

        RefactoringStatus conditionCheckingStatus = create.getConditionCheckingStatus();
        ProblemReport problemReport = new ProblemReport();
        if (conditionCheckingStatus.getSeverity() != RefactoringStatus.OK) {
            problemReport.addProblems(conditionCheckingStatus.getEntries());
        }
        return problemReport;
    }

    public static void undoRefactoring(Change change) throws CoreException {
        PerformChangeOperation performUndo = new PerformChangeOperation(change);
        IWorkspace workspace = ResourcesPlugin.getWorkspace();
        workspace.run(performUndo, EmptyProgressMonitor.inst());
    }

    public static Change getUndoChange() {
        return undoChange;
    }

}
