package coco.task.etc;

import org.eclipse.jdt.core.dom.ASTVisitor;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.Modifier;
import org.eclipse.jdt.core.dom.TypeDeclaration;

import coco.CoCoASTV;

/**
 * This is a helper {@link ASTVisitor} to check if a compilation unit that is
 * being visited should be ignored.
 * 
 * At this moment every compilation unit that does not contain at least one
 * {@link TypeDeclaration}, different from interface or abstract class, is
 * ignored.
 * 
 * Note that the visitor does not extend {@link CoCoASTV} because it is
 * independent.
 * 
 * @author Milos Gligoric (milos.gligoric@gmail.com)
 * @author Rohan Sharma (sharma.rohan1990@gmail.com)
 */
public class CheckIfValidCUASTV extends ASTVisitor {

    /** True if compilation unit is valid, i.e., it should be visited */
    private boolean isValid;

    public boolean isValid() {
        return isValid;
    }

    // ASTVisitor interface

    @Override
    public boolean visit(CompilationUnit node) {
        isValid = true;
        checkAllInterfaces(node);
        // the following code is used to ignore compilation units that have only
        // abstract classes.
        // checkAllAbstracts(node);
        // IMPORTANT: change to "true" if more checks are added for other nodes
        return false;
    }

    // PRIVATE

    private void checkAllInterfaces(CompilationUnit node) {
        isValid = false;
        for (Object obj : node.types()) {
            if (obj instanceof TypeDeclaration) {
                TypeDeclaration type = (TypeDeclaration) obj;
                if (!type.isInterface()) {
                    isValid = true;
                    break;
                }
            }
        }
    }

    @SuppressWarnings("unused")
    private void checkAllAbstracts(CompilationUnit node) {
        isValid = false;
        for (Object obj : node.types()) {
            if (obj instanceof TypeDeclaration) {
                TypeDeclaration type = (TypeDeclaration) obj;
                if (!type.isInterface() && !Modifier.isAbstract(type.getModifiers())) {
                    isValid = true;
                    break;
                }
            }
        }
    }

}
