package coco.task.etc;

import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTParser;
import org.eclipse.jdt.core.dom.Comment;
import org.eclipse.jdt.core.dom.CompilationUnit;

import coco.task.EmptyProgressMonitor;
import coco.task.ITask;

/**
 * Remove all comments from compilation unit(s).
 * 
 * @author Milos Gligoric (milos.gligoric@gmail.com)
 * @author Rohan Sharma (sharma.rohan1990@gmail.com)
 */
public class RemoveCommentsTask implements ITask {

    private IProject project;

    public RemoveCommentsTask(IProject project) {
        this.project = project;
    }

    @Override
    public void perform(ICompilationUnit icu) throws Exception {
        ASTParser parser = ASTParser.newParser(AST.JLS3);
        parser.setSource(icu);
        CompilationUnit cu = (CompilationUnit) parser.createAST(EmptyProgressMonitor.inst());
        @SuppressWarnings("unchecked")
        List<Comment> commentList = (List<Comment>) cu.getCommentList();
        StringBuilder code = new StringBuilder(icu.getSource());
        // do backward
        for (int i = commentList.size() - 1; i >= 0; i--) {
            Comment comment = commentList.get(i);
            code.delete(comment.getStartPosition(), comment.getStartPosition() + comment.getLength());
        }
        icu.getBuffer().setContents(code.toString());
        icu.save(EmptyProgressMonitor.inst(), true);
        project.build(IncrementalProjectBuilder.FULL_BUILD, EmptyProgressMonitor.inst());
    }

}
