package coco.task.etc;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.jdt.core.compiler.IProblem;

import coco.CoCoASTV;
import coco.util.ICUCompiler.ICUCompilerResult;

/**
 * Collect start position of each error that will be removed because it matches
 * one of the specified types.
 * 
 * @author Milos Gligoric (milos.gligoric@gmail.com)
 * @author Rohan Sharma (sharma.rohan1990@gmail.com)
 */
public abstract class RemoveErrorsASTV extends CoCoASTV {

    private final ICUCompilerResult compilerResult;
    private final String errMsgContains;
    protected final List<Integer> errorCodeStarts;

    public RemoveErrorsASTV(ICUCompilerResult compilerResult, String errMsgContains) {
        this.compilerResult = compilerResult;
        this.errMsgContains = errMsgContains;
        this.errorCodeStarts = findStartPositionErrorStatements();
    }

    protected List<Integer> findStartPositionErrorStatements() {
        List<Integer> statementsWithErrors = new LinkedList<Integer>();
        for (IProblem error : compilerResult.getErrors()) {
            if (error.getMessage().contains(errMsgContains)) {
                statementsWithErrors.add(error.getSourceStart());
            }
        }
        return statementsWithErrors;
    }

}
