package coco.task.etc;

import org.eclipse.jdt.core.dom.MethodDeclaration;

import coco.util.ICUCompiler.ICUCompilerResult;

/**
 * Removing errors that may exist after instrumentation.
 * 
 * @author Milos Gligoric (milos.gligoric@gmail.com)
 * @author Rohan Sharma (sharma.rohan1990@gmail.com)
 */
public class RemoveIncorrectContextASTV extends RemoveErrorsASTV {

    // IMPORTANT: compiler dependent!
    private static final String ERR_MSG = "Cannot make a static reference to the non-static type";

    public RemoveIncorrectContextASTV(ICUCompilerResult compilerResult) {
        super(compilerResult, ERR_MSG);
    }

    // ASTVisitor interface

    @Override
    public boolean visit(MethodDeclaration node) {
        // remove all nodes that belong to identified error lines
        for (int errorCodeStart : errorCodeStarts) {
            if (node.getStartPosition() <= errorCodeStart
                    && (node.getStartPosition() + node.getLength()) >= errorCodeStart) {
                node.delete();
                return false;
            }
        }
        return true;
    }
}
