package coco.task.extract.pct;

import java.util.List;
import java.util.Map;

import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.AnonymousClassDeclaration;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.TypeDeclaration;

import coco.CoCoASTV;

/**
 * Add predicates (code) in the compilation units. Predicates are already
 * extracted and passed as the argument to this visitor.
 * 
 * @author Milos Gligoric (milos.gligoric@gmail.com)
 * @author Rohan Sharma (sharma.rohan1990@gmail.com)
 * 
 * @see ExtractPredicatesTask
 * @see ConstructPredicatesASTV
 */
public class AddPredicatesASTV extends CoCoASTV {

    /** Mapping from type sequence number to predicates */
    private Map<Integer, List<MethodDeclaration>> typeToPredicates;

    /** Next type in order */
    private int nextTypeInSeq = 0;

    public AddPredicatesASTV(ConstructPredicatesASTV makePredicates) {
        this.typeToPredicates = makePredicates.getTypeToPredicates();
    }

    // ASTVisitor interface

    @Override
    public boolean visit(AnonymousClassDeclaration node) {
        return visitType(node.bodyDeclarations());
    }

    @Override
    public boolean oVisit(TypeDeclaration node) {
        return visitType(node.bodyDeclarations());
    }

    // PRIVATE

    @SuppressWarnings({ "unchecked", "rawtypes" })
    private boolean visitType(List bodyDeclarations) {
        nextTypeInSeq++;
        List<MethodDeclaration> predicates = typeToPredicates.get(nextTypeInSeq);
        if (predicates != null) {
            for (MethodDeclaration predicate : predicates) {
                bodyDeclarations.add(ASTNode.copySubtree(ast(), predicate));
            }
        }
        return true;
    }

}
