package coco.task.extract.pct;

import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.FieldAccess;
import org.eclipse.jdt.core.dom.MethodInvocation;
import org.eclipse.jdt.core.dom.Name;
import org.eclipse.jdt.core.dom.QualifiedName;
import org.eclipse.jdt.core.dom.SuperMethodInvocation;
import org.eclipse.jdt.core.dom.ThisExpression;
import org.eclipse.jdt.internal.core.ResolvedBinaryField;
import org.eclipse.jdt.internal.core.ResolvedBinaryType;
import org.eclipse.jdt.internal.core.ResolvedSourceField;
import org.eclipse.jdt.internal.core.ResolvedSourceType;

/**
 * @author Milos Gligoric (milos.gligoric@gmail.com)
 * @author Rohan Sharma (sharma.rohan1990@gmail.com)
 */
@SuppressWarnings("restriction")
public class DetectNullChecksASTV extends DetectXASTV {

    public DetectNullChecksASTV(DetectPureMethodsASTV pureVisitor) {
        super(pureVisitor);
    }

    // ASTVisitor interface

    @Override
    public boolean visit(QualifiedName node) {
        if (!isAccessUsingClass(node.getQualifier())) {
            IJavaElement javaElement = node.resolveBinding().getJavaElement();
            if (javaElement instanceof ResolvedSourceField || javaElement instanceof ResolvedBinaryField) {
                visitAccess(node.getQualifier());
            }
        }
        return true;
    }

    @Override
    public boolean visit(FieldAccess node) {
        if (!isAccessUsingClass(node.getExpression())) {
            visitAccess(node.getExpression());
        }
        return true;
    }

    @Override
    public boolean visit(MethodInvocation node) {
        Expression expr = node.getExpression();
        if (expr != null && !isAccessUsingClass(expr) && isNotThis(expr) && isNotSuper(expr)) {
            visitAccess(node.getExpression());
        }
        return true;
    }

    // PRIVATE

    private boolean isNotThis(Expression expr) {
        return !(expr instanceof ThisExpression);
    }

    private boolean isNotSuper(Expression expr) {
        return !(expr instanceof SuperMethodInvocation);
    }

    private boolean isAccessUsingClass(Expression expr) {
        if (expr instanceof Name) {
            IJavaElement qualifierJavaElement = ((Name) expr).resolveBinding().getJavaElement();
            if (qualifierJavaElement instanceof ResolvedSourceType
                    || qualifierJavaElement instanceof ResolvedBinaryType) {
                return true;
            }
        }
        return false;
    }

    private void visitAccess(Expression expr) {
        getCallGraphASTVisitor().setCurrentType(stackOfTypes.peek());
        expr.accept(getCallGraphASTVisitor());
        if (!getCallGraphASTVisitor().isPure()) {
            return;
        }
        internalCounter++;
        if (internalCounter == currentConditionId && !hasCondition) {
            hasCondition = true;
            expression = expr;
            currentConditionId++;
        }
    }

}
