package coco.task.extract.pct;

import java.util.Stack;

import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.dom.AnonymousClassDeclaration;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.TypeDeclaration;

import coco.CoCoASTV;
import coco.IResettableV;
import coco.task.extract.pct.DetectPureMethodsASTV.CallGraphASTVisitor;

/**
 * @author Milos Gligoric (milos.gligoric@gmail.com)
 * @author Rohan Sharma (sharma.rohan1990@gmail.com)
 */
abstract class DetectXASTV extends CoCoASTV implements IResettableV {

    /** Result of purity analysis to check each expression/condition */
    private final CallGraphASTVisitor callGraphV;

    protected final Stack<String> stackOfTypes;

    protected int currentConditionId;
    protected int internalCounter;
    protected boolean hasCondition;
    protected Expression expression;

    public DetectXASTV(DetectPureMethodsASTV detectPureMethodsV) {
        this.currentConditionId = 1;
        this.callGraphV = detectPureMethodsV.getCallGraphASTVisitor();
        this.stackOfTypes = new Stack<String>();
    }

    public int getCurrentConditionId() {
        return currentConditionId - 1;
    }

    public boolean hasCondition() {
        return hasCondition;
    }

    public int getStartPosition() {
        return expression.getStartPosition();
    }

    public Expression getExpression() {
        return expression;
    }

    public int getLength() {
        return expression.getLength();
    }

    @Override
    public void reset() {
        hasCondition = false;
        internalCounter = 0;
        expression = null;
    }

    protected CallGraphASTVisitor getCallGraphASTVisitor() {
        return callGraphV;
    }

    // ASTVisitor interface

    @Override
    public final boolean visit(AnonymousClassDeclaration node) {
        IType type = (IType) node.resolveBinding().getJavaElement();
        try {
            String typeAsString = type.getFullyQualifiedName() + type.getSourceRange().getOffset();
            stackOfTypes.push(typeAsString);
        } catch (JavaModelException e) {
            e.printStackTrace();
        }
        return true;
    }

    @Override
    public final void endVisit(AnonymousClassDeclaration node) {
        stackOfTypes.pop();
    }

    @Override
    public final boolean oVisit(TypeDeclaration node) {
        IType type = (IType) node.resolveBinding().getJavaElement();
        try {
            String typeAsString = type.getFullyQualifiedName() + type.getSourceRange().getOffset();
            assert (typeAsString != null);
            stackOfTypes.push(typeAsString);
        } catch (JavaModelException e) {
            e.printStackTrace();
        }
        return true;
    }

    @Override
    public final void oEndVisit(TypeDeclaration node) {
        stackOfTypes.pop();
    }
}
