package coco.task.instantiate;

import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.dom.AnonymousClassDeclaration;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.TypeDeclaration;

import coco.CoCoASTV;
import coco.util.CoCoUtil;

/**
 * Collect predicates for each type.
 * 
 * TODO: This class should be merged with previous task in sequence.
 * 
 * @author Milos Gligoric (milos.gligoric@gmail.com)
 * @author Rohan Sharma (sharma.rohan1990@gmail.com)
 */
public class CollectTypePredicatesMappingASTV extends CoCoASTV {

    private final ITypePredicatesMap typePredicateMap;

    public CollectTypePredicatesMappingASTV(ICompilationUnit icu) {
        this.typePredicateMap = new ITypePredicatesMap();
    }

    public final ITypePredicatesMap getTypePredicateMap() {
        return typePredicateMap;
    }

    // ASTVisitor interface

    @Override
    public boolean visit(AnonymousClassDeclaration node) {
        typePredicateMap.beginLevel();
        return true;
    }

    @Override
    public final void endVisit(AnonymousClassDeclaration node) {
        typePredicateMap.endLevel();
    }

    @Override
    public boolean oVisit(TypeDeclaration node) {
        typePredicateMap.beginLevel();
        return true;
    }

    @Override
    public final void oEndVisit(TypeDeclaration node) {
        typePredicateMap.endLevel();
    }

    @Override
    public final boolean visit(MethodDeclaration node) {
        if (!CoCoUtil.isPredicate(node)) {
            return true;
        }
        IType t = CoCoUtil.getDeclaringIType(node);
        IMethod m = CoCoUtil.getIMethod(node);
        typePredicateMap.add(t, m);
        return true;
    }

}
