package coco.task.instantiate;

import org.eclipse.jdt.core.dom.Block;
import org.eclipse.jdt.core.dom.EnhancedForStatement;
import org.eclipse.jdt.core.dom.ForStatement;
import org.eclipse.jdt.core.dom.IfStatement;
import org.eclipse.jdt.core.dom.LabeledStatement;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.Statement;
import org.eclipse.jdt.core.dom.SwitchStatement;
import org.eclipse.jdt.core.dom.WhileStatement;

import coco.CoCoASTV;
import coco.util.CoCoUtil;

/**
 * Enclose all loops in blocks, e.g., <code>while(condition){...}</code> becomes
 * <code>{while(condition){...}}</code>.
 * 
 * @author Milos Gligoric (milos.gligoric@gmail.com)
 * @author Rohan Sharma (sharma.rohan1990@gmail.com)
 */
public class EncloseLoopsInBlockASTV extends CoCoASTV {

    // ASTVisitor interface

    @Override
    public boolean visit(MethodDeclaration node) {
        if (CoCoUtil.isCondition(node) || CoCoUtil.isPredicate(node) || node.getBody() == null) {
            return false;
        }
        return true;
    }

    @SuppressWarnings("unchecked")
    @Override
    public void endVisit(Block node) {
        for (int i = 0; i < node.statements().size(); i++) {
            Statement stmt = (Statement) node.statements().get(i);
            if (stmt instanceof WhileStatement || stmt instanceof IfStatement || stmt instanceof ForStatement
                    || stmt instanceof EnhancedForStatement || stmt instanceof SwitchStatement
                    || stmt instanceof LabeledStatement) {
                Block around = ast().newBlock();
                node.statements().set(i, around);
                stmt.delete();
                around.statements().add(stmt);
            }
        }
    }

}
