package coco.task.instantiate;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.text.edits.MalformedTreeException;

import coco.CoCoInstrumentationOpt;
import coco.log.Log;
import coco.task.ITask;
import coco.task.etc.RemoveUnreachableBlocksASTV;
import coco.util.ASTVHelper;
import coco.util.ICUCompiler;
import coco.util.ICUCompiler.ICUCompilerResult;

/**
 * Task to insert {@link Block} statements at each basic blocks.
 * 
 * @author Milos Gligoric (milos.gligoric@gmail.com)
 * @author Rohan Sharma (sharma.rohan1990@gmail.com)
 */
public class InsertBasicBlocksTask implements ITask {

    @SuppressWarnings("unused")
    private CoCoInstrumentationOpt args;
    private IProject project;

    public InsertBasicBlocksTask(CoCoInstrumentationOpt args, IProject project) {
        this.args = args;
        this.project = project;
    }

    @Override
    public void perform(ICompilationUnit icu) throws Exception {
        // enclose loops
        EncloseLoopsInBlockASTV encloseLoopsInBlockASTVisitor = new EncloseLoopsInBlockASTV();
        ASTVHelper.acceptVisitor(project, icu, encloseLoopsInBlockASTVisitor, false);

        // insert basic blocks
        performInsertBasicBlocks(icu);

        // remove errors after adding blocks
        ICUCompilerResult compilerResult = ICUCompiler.inst().compile(icu);
        if (!compilerResult.isCompilationSuccessful()) {
            Log.e(getClass().getName() + " " + compilerResult.getErrors());

            RemoveUnreachableBlocksASTV removeUnreachableBlocks = new RemoveUnreachableBlocksASTV(compilerResult);
            ASTVHelper.acceptVisitor(project, icu, removeUnreachableBlocks, false);
        }
    }

    private void performInsertBasicBlocks(ICompilationUnit icu) throws MalformedTreeException, BadLocationException,
            CoreException {
        InsertBasicBlockASTV visitor = new InsertBasicBlockASTV();
        do {
            visitor.advance();
            ASTVHelper.acceptVisitor(project, icu, visitor, false);
        } while (visitor.hasNextLevel());
    }

}
