package coco.task.instantiate;

import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.SingleVariableDeclaration;
import org.eclipse.jdt.core.dom.Type;

import coco.CoCoASTV;

/**
 * Visitor that rebuilds a list of parameters to replace "type name[]" with
 * "type[] name". Assumes that eclipse builders build "type[] name".
 * 
 * @author Milos Gligoric (milos.gligoric@gmail.com)
 * @author Rohan Sharma (sharma.rohan1990@gmail.com)
 */
public class MoveExtraDimensionsASTV extends CoCoASTV {

    @SuppressWarnings("unchecked")
    @Override
    public boolean visit(MethodDeclaration node) {
        for (Object obj : node.parameters()) {
            SingleVariableDeclaration svd = (SingleVariableDeclaration) obj;
            SingleVariableDeclaration newSvd = ast().newSingleVariableDeclaration();
            // reset dimension (type will be set to appropriate dimension)
            newSvd.setExtraDimensions(0);
            newSvd.setFlags(svd.getFlags());
            // NOP: newSvd.setInitializer(initializer)
            for (Object m : svd.modifiers()) {
                newSvd.modifiers().add(ASTNode.copySubtree(ast(), (ASTNode) m));
            }
            // newSvd.setModifiers(modifiers)
            newSvd.setName((SimpleName) ASTNode.copySubtree(ast(), svd.getName()));
            if (svd.getExtraDimensions() > 0) {
                // TODO: check if it is array type and add extra dimensions
                newSvd.setType(ast().newArrayType((Type) ASTNode.copySubtree(ast(), svd.getType()),
                        svd.getExtraDimensions()));
            } else {
                newSvd.setType((Type) ASTNode.copySubtree(ast(), svd.getType()));
            }
            newSvd.setVarargs(svd.isVarargs());
            rewrite().replace(svd, newSvd, null);
        }
        return true;
    }
}
