package coco.task.instantiate.bc;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.Block;
import org.eclipse.jdt.core.dom.DoStatement;
import org.eclipse.jdt.core.dom.EnhancedForStatement;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.ForStatement;
import org.eclipse.jdt.core.dom.IfStatement;
import org.eclipse.jdt.core.dom.MethodInvocation;
import org.eclipse.jdt.core.dom.Statement;
import org.eclipse.jdt.core.dom.SwitchCase;
import org.eclipse.jdt.core.dom.SwitchStatement;
import org.eclipse.jdt.core.dom.WhileStatement;

import coco.CoCoInstrumentationOpt;
import coco.task.instantiate.ITypePredicatesMap;
import coco.task.instantiate.InstantiateASTV;
import coco.util.ASTBuilder;
import coco.util.CoCoASTBuilder;

/**
 * Code instrumentation for measuring BC.
 * 
 * @author Milos Gligoric (milos.gligoric@gmail.com)
 * @author Rohan Sharma (sharma.rohan1990@gmail.com)
 */
public class InstantiateBC extends InstantiateASTV {

    private static String TOUCH_BRANCH_METHOD = CoCoInstrumentationOpt.TOUCH_BRANCH_METHOD;

    public InstantiateBC(CoCoInstrumentationOpt args, ITypePredicatesMap typePredicateMap, long pointId) {
        super(args, typePredicateMap, pointId);
    }

    // ASTVisitor interface

    @Override
    public boolean visit(IfStatement node) {
        insertPredicate(node.getThenStatement());
        insertPredicate(node.getElseStatement());
        // insertPredicatesAfter(node);
        return true;
    }

    @Override
    public void extendVisit(ForStatement node) {
        insertPredicate(node.getBody());
    }

    @Override
    public void extendEndVisit(ForStatement node) {
        insertPredicatesAfter(node);
    }

    @Override
    public void extendVisit(EnhancedForStatement node) {
        insertPredicate(node.getBody());
        insertPredicatesAfter(node);
    }

    @Override
    public boolean visit(WhileStatement node) {
        insertPredicate(node.getBody());
        insertPredicatesAfter(node);
        return true;
    }

    @Override
    public boolean visit(DoStatement node) {
        insertPredicate(node.getBody());
        insertPredicatesAfter(node);
        return true;
    }

    @Override
    public boolean visit(SwitchStatement node) {
        for (int i = 0; i < node.statements().size(); i++) {
            Statement s = (Statement) node.statements().get(i);
            if (!(s instanceof SwitchCase)) {
                insertPredicate(s);
            }
        }
        // TODO: Should default branch be added?
        // insertPredicatesAfter(node);
        return true;
    }

    // PROTECTED

    @Override
    protected List<?> getStatements(Statement node) {
        return ((Block) node).statements();
    }

    @SuppressWarnings("rawtypes")
    @Override
    protected int whereToInsert(List statements, Statement node) {
        return 0;
    }

    @SuppressWarnings({ "unchecked", "rawtypes" })
    @Override
    protected void insertPredicate(Statement node) {
        try {
            List<Expression> predicateInvocations = new LinkedList<Expression>();
            pointId++;
            MethodInvocation instInvocation = CoCoASTBuilder.createInstMethodInvocation(ast());
            predicateInvocations.add(0, ast().newNumberLiteral(Long.toString(pointId)));
            MethodInvocation mi = ASTBuilder.createMethodInvocation(ast(), instInvocation, TOUCH_BRANCH_METHOD,
                    predicateInvocations.toArray(new Expression[0]));
            List statements = getStatements(node);
            int index = whereToInsert(statements, node);
            statements.add(index, ast().newExpressionStatement(mi));
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    // PRIVATE

    private void insertPredicatesAfter(ASTNode node) {
        Block block = (Block) node.getParent();
        int index = block.statements().indexOf(node);
        if (index + 1 < block.statements().size()) {
            // maybe we have removed after block (unreachable code)
            Block after = (Block) block.statements().get(index + 1);
            insertPredicate(after);
        }
    }
}
