package coco.task.instantiate.fase;

import java.util.List;

import org.eclipse.jdt.core.dom.ASTMatcher;
import org.eclipse.jdt.core.dom.Block;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.ExpressionStatement;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.MethodInvocation;
import org.eclipse.jdt.core.dom.Statement;

import coco.CoCoInstrumentationOpt;
import coco.task.etc.RemoveExistingInstrumentationTask;
import coco.task.instantiate.ITypePredicatesMap;
import coco.task.instantiate.InstantiateASTV;
import coco.util.CoCoASTBuilder;
import coco.util.CoCoUtil;

/**
 * Instantiate predicates at the same places as it was done for FASE 2011
 * (http://mir.cs.illinois.edu/coverage/). This is utility class. It requires
 * {@link RemoveExistingInstrumentationTask} to be included in the list of
 * tasks.
 * 
 * @author Milos Gligoric (milos.gligoric@gmail.com)
 * @author Rohan Sharma (sharma.rohan1990@gmail.com)
 */
public class InstantiateFASE extends InstantiateASTV {

    private Expression COCO_RUNTIME_INST_INVOCATION;

    public InstantiateFASE(CoCoInstrumentationOpt args, ITypePredicatesMap typePredicateMap, long pointId) {
        super(args, typePredicateMap, pointId);
    }

    // ASTVisitor interface

    @SuppressWarnings("deprecation")
    @Override
    public boolean visit(ExpressionStatement node) {
        if (CoCoUtil.isConditionInvocation(node)) {
            return false;
        }
        if (node.getExpression() instanceof MethodInvocation) {
            MethodInvocation methodInvocation = (MethodInvocation) node.getExpression();
            if (methodInvocation != null && methodInvocation.subtreeMatch(new ASTMatcher(), getCoCoRuntimeInst())) {
                insertPredicate(node);
                node.delete();
            }
        }
        return true;
    }

    @Override
    public boolean extendVisit(MethodDeclaration node) {
        return !CoCoUtil.isCondition(node);
    }

    // PROTECTED

    @Override
    protected List<?> getStatements(Statement node) {
        Block parent = (Block) node.getParent();
        return parent.statements();
    }

    @Override
    @SuppressWarnings("rawtypes")
    protected int whereToInsert(List statements, Statement node) {
        return statements.indexOf(node);
    }

    // PRIVATE

    private Expression getCoCoRuntimeInst() {
        // lazy initialization
        if (COCO_RUNTIME_INST_INVOCATION == null) {
            COCO_RUNTIME_INST_INVOCATION = CoCoASTBuilder.createInstMethodInvocation(ast());
        }
        return COCO_RUNTIME_INST_INVOCATION;
    }

}
