package coco.task.instantiate.pctst;

import java.util.List;

import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.AssertStatement;
import org.eclipse.jdt.core.dom.Block;
import org.eclipse.jdt.core.dom.BreakStatement;
import org.eclipse.jdt.core.dom.ContinueStatement;
import org.eclipse.jdt.core.dom.DoStatement;
import org.eclipse.jdt.core.dom.EnhancedForStatement;
import org.eclipse.jdt.core.dom.ExpressionStatement;
import org.eclipse.jdt.core.dom.ForStatement;
import org.eclipse.jdt.core.dom.IfStatement;
import org.eclipse.jdt.core.dom.LabeledStatement;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.ReturnStatement;
import org.eclipse.jdt.core.dom.Statement;
import org.eclipse.jdt.core.dom.SwitchStatement;
import org.eclipse.jdt.core.dom.SynchronizedStatement;
import org.eclipse.jdt.core.dom.ThrowStatement;
import org.eclipse.jdt.core.dom.TryStatement;
import org.eclipse.jdt.core.dom.VariableDeclarationStatement;
import org.eclipse.jdt.core.dom.WhileStatement;

import coco.CoCoInstrumentationOpt;
import coco.task.instantiate.ITypePredicatesMap;
import coco.task.instantiate.InstantiateASTV;
import coco.util.CoCoUtil;

/**
 * Code instrumentation for measuring PCT(ST).
 * 
 * @author Milos Gligoric (milos.gligoric@gmail.com)
 * @author Rohan Sharma (sharma.rohan1990@gmail.com)
 */
public class InstantiatePCTST extends InstantiateASTV {

    public InstantiatePCTST(CoCoInstrumentationOpt args, ITypePredicatesMap typePredicateMap, long pointId) {
        super(args, typePredicateMap, pointId);
    }

    // ASTVisitor interface

    @Override
    public boolean visit(AssertStatement node) {
        insertPredicate(node);
        return true;
    }

    @Override
    public boolean visit(BreakStatement node) {
        insertPredicate(node);
        return true;
    }

    @Override
    public boolean visit(ContinueStatement node) {
        insertPredicate(node);
        return true;
    }

    @Override
    public boolean visit(DoStatement node) {
        insertPredicate(node);
        return true;
    }

    // NOTTODO: remove
    // @Override
    // public boolean visit(EmptyStatement node) {
    // insertPredicate(node);
    // return true;
    // }

    @Override
    public void extendEndVisit(EnhancedForStatement node) {
        insertPredicate(node);
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean visit(ExpressionStatement node) {
        if (CoCoUtil.isConditionInvocation(node)) {
            return false;
        }
        insertPredicate(node);
        return true;
    }

    @Override
    public void extendEndVisit(ForStatement node) {
        insertPredicate(node);
    }

    @Override
    public boolean visit(IfStatement node) {
        insertPredicate(node);
        return true;
    }

    @Override
    public boolean visit(LabeledStatement node) {
        insertPredicate(node);
        return true;
    }

    @Override
    public boolean visit(ReturnStatement node) {
        insertPredicate(node);
        return true;
    }

    @Override
    public boolean visit(SwitchStatement node) {
        insertPredicate(node);
        return true;
    }

    @Override
    public boolean visit(SynchronizedStatement node) {
        insertPredicate(node);
        return true;
    }

    @Override
    public boolean visit(ThrowStatement node) {
        insertPredicate(node);
        return true;
    }

    @Override
    public boolean visit(TryStatement node) {
        insertPredicate(node);
        return true;
    }

    // NOTTODO: remove
    // @Override
    // public boolean visit(TypeDeclarationStatement node) {
    // insertPredicate(node);
    // return true;
    // }

    @Override
    public boolean visit(VariableDeclarationStatement node) {
        insertPredicate(node);
        return true;
    }

    @Override
    public boolean visit(WhileStatement node) {
        insertPredicate(node);
        return true;
    }

    @Override
    public boolean extendVisit(MethodDeclaration node) {
        return !CoCoUtil.isCondition(node);
    }

    // PROTECTED

    @Override
    protected List<?> getStatements(Statement node) {
        ASTNode parent = node.getParent();
        // while (!(parent instanceof Block)) {
        // parent = parent.getParent();
        // }
        Block block = (Block) parent;
        return block.statements();
    }

    @Override
    protected int whereToInsert(List<?> statements, Statement node) {
        return statements.indexOf(node);
    }

    /**
     * Need to override to check if node.parent is LabeledStatement in which
     * case we should ignore instrumentation.
     */
    @Override
    protected void insertPredicate(Statement node, String touchMethodName) {
        // if node is LabeledStatement we ignore it because we will instantiate
        // for LabeledStatement
        if (node.getParent() instanceof LabeledStatement) {
            return;
        }
        super.insertPredicate(node, touchMethodName);
    }
}
