package coco.task.instantiate.sc;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.LabeledStatement;
import org.eclipse.jdt.core.dom.MethodInvocation;
import org.eclipse.jdt.core.dom.Statement;

import coco.CoCoInstrumentationOpt;
import coco.task.instantiate.ITypePredicatesMap;
import coco.task.instantiate.pctst.InstantiatePCTST;
import coco.util.ASTBuilder;
import coco.util.CoCoASTBuilder;

/**
 * Code instrumentation for measuring SC.
 * 
 * @author Milos Gligoric (milos.gligoric@gmail.com)
 * @author Rohan Sharma (sharma.rohan1990@gmail.com)
 */
public class InstantiateSC extends InstantiatePCTST {

    private static String TOUCH_STATEMENT_METHOD = CoCoInstrumentationOpt.TOUCH_METHOD;

    public InstantiateSC(CoCoInstrumentationOpt args, ITypePredicatesMap typePredicateMap, long pointId) {
        super(args, typePredicateMap, pointId);
    }

    // PROTECTED

    @SuppressWarnings({ "unchecked", "rawtypes" })
    @Override
    protected void insertPredicate(Statement node) {
        try {
            // if node is child of LabeledStatement we ignore it because there
            // will be instantiation for LabeledStatement
            if (node.getParent() instanceof LabeledStatement) {
                return;
            }
            List<Expression> predicateInvocations = new LinkedList<Expression>();
            pointId++;
            MethodInvocation instInvocation = CoCoASTBuilder.createInstMethodInvocation(ast());
            predicateInvocations.add(0, ast().newNumberLiteral(Long.toString(pointId)));
            MethodInvocation mi = ASTBuilder.createMethodInvocation(ast(), instInvocation, TOUCH_STATEMENT_METHOD,
                    predicateInvocations.toArray(new Expression[0]));
            List statements = getStatements(node);
            int index = whereToInsert(statements, node);
            statements.add(index, ast().newExpressionStatement(mi));
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

}
