package coco.task.renamevars;

import java.util.Set;

import org.eclipse.jdt.core.IField;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.ILocalVariable;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.IVariableBinding;
import org.eclipse.jdt.core.dom.SingleVariableDeclaration;
import org.eclipse.jdt.core.dom.VariableDeclaration;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;

import coco.CoCoASTV;

/**
 * @author Milos Gligoric (milos.gligoric@gmail.com)
 * @author Rohan Sharma (sharma.rohan1990@gmail.com)
 */
public class RenameVarsASTV extends CoCoASTV {

    /** Names that have been seen */
    private Set<String> names;

    private CompilationUnit cu;
    private VariableDeclaration variableDeclaration;
    private IJavaElement iJavaElement;

    private int nextToMatch;
    private int currentTraversalCounter;

    public RenameVarsASTV(Set<String> names) {
        this.names = names;
    }

    public CompilationUnit getCompilationUnit() {
        return cu;
    }

    public boolean hasVariable() {
        return iJavaElement != null;
    }

    public IJavaElement getElement() {
        return iJavaElement;
    }

    public void advanceId() {
        nextToMatch++;
        currentTraversalCounter = 0;
        iJavaElement = null;
    }

    public int getId() {
        return nextToMatch;
    }

    public VariableDeclaration getParentVariableDeclaration() {
        return variableDeclaration;
    }

    // ASTVisitor interface

    @Override
    public boolean visit(SingleVariableDeclaration node) {
        variableDeclaration(node);
        return true;
    }

    @Override
    public boolean visit(VariableDeclarationFragment node) {
        variableDeclaration(node);
        return true;
    }

    @Override
    public boolean oVisit(CompilationUnit node) {
        super.oVisit(node);
        cu = node;
        return true;
    }

    // PRIVATE

    private void variableDeclaration(VariableDeclaration node) {
        currentTraversalCounter++;
        IVariableBinding binding = node.resolveBinding();
        IJavaElement element = binding.getJavaElement();

        if ((element instanceof ILocalVariable || element instanceof IField) && currentTraversalCounter == nextToMatch) {
            if (names.add(element.getElementName())) {
                nextToMatch++;
                return;
            }
            variableDeclaration = node;
            iJavaElement = element;
        }
    }

}
