package coco.util;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTParser;
import org.eclipse.jdt.core.dom.ASTVisitor;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.IDocument;
import org.eclipse.text.edits.MalformedTreeException;
import org.eclipse.text.edits.TextEdit;
import org.eclipse.text.edits.UndoEdit;

import coco.CoCoASTV;
import coco.RepeatCoverageASTV;
import coco.task.EmptyProgressMonitor;

/**
 * Extracted code for an AST traversal, which assumes/saves modifications.
 * 
 * @author Milos Gligoric (milos.gligoric@gmail.com)
 * @author Rohan Sharma (sharma.rohan1990@gmail.com)
 */
@SuppressWarnings("deprecation")
public class ASTVHelper {

    public static void acceptVisitorRepeat(IProject project, ICompilationUnit icu, RepeatCoverageASTV visitor,
            boolean resolveBindings) throws MalformedTreeException, BadLocationException, CoreException {
        while (true) {
            visitor.advance();
            CompilationUnit cu = createCU(icu, resolveBindings);
            cu.recordModifications();

            IDocument doc = new Document(icu.getSource());
            cu.accept(visitor);
            if (!visitor.isEncountered()) {
                break;
            }
            TextEdit textEdits = cu.rewrite(doc, null);
            applyModifications(project, icu, doc, textEdits);
        }
    }

    public static void acceptNoModify(IProject project, ICompilationUnit icu, ASTVisitor visitor,
            boolean resolveBindings) throws JavaModelException {
        CompilationUnit cu = createCU(icu, resolveBindings);

        cu.accept(visitor);
    }

    public static UndoEdit acceptVisitor(IProject project, ICompilationUnit icu, ASTVisitor visitor,
            boolean resolveBindings) throws MalformedTreeException, BadLocationException, CoreException {
        CompilationUnit cu = createCU(icu, resolveBindings);
        cu.recordModifications();

        IDocument doc = new Document(icu.getSource());
        cu.accept(visitor);
        TextEdit textEdits = cu.rewrite(doc, null);
        return applyModifications(project, icu, doc, textEdits);
    }

    public static void acceptVisitorRewrite(IProject project, ICompilationUnit icu, CoCoASTV visitor,
            boolean resolveBindings) throws MalformedTreeException, BadLocationException, CoreException {
        CompilationUnit cu = createCU(icu, resolveBindings);

        IDocument doc = new Document(icu.getSource());
        cu.accept(visitor);
        TextEdit textEdits = visitor.rewrite().rewriteAST(doc, null);
        applyModifications(project, icu, doc, textEdits);
    }

    // PRIVATE

    private static CompilationUnit createCU(ICompilationUnit icu, boolean resolveBindings) {
        ASTParser parser = ASTParser.newParser(AST.JLS3);
        parser.setResolveBindings(resolveBindings);
        parser.setSource(icu);
        return (CompilationUnit) parser.createAST(EmptyProgressMonitor.inst());
    }

    private static UndoEdit applyModifications(IProject project, ICompilationUnit icu, IDocument doc, TextEdit textEdits)
            throws MalformedTreeException, BadLocationException, CoreException {
        UndoEdit undoEdit = textEdits.apply(doc);
        icu.getBuffer().setContents(doc.get());
        icu.save(EmptyProgressMonitor.inst(), true);
        project.build(IncrementalProjectBuilder.FULL_BUILD, EmptyProgressMonitor.inst());
        return undoEdit;
    }

}
