package coco.util;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IProblemRequestor;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.compiler.IProblem;

/**
 * {@link ICompilationUnit} "compiler".
 * 
 * @author Milos Gligoric (milos.gligoric@gmail.com)
 * @author Rohan Sharma (sharma.rohan1990@gmail.com)
 */
public class ICUCompiler implements IProblemRequestor {

    public static class ICUCompilerResult {
        private List<IProblem> errors = new LinkedList<IProblem>();

        public List<IProblem> getErrors() {
            return errors;
        }

        public boolean isCompilationSuccessful() {
            return errors.size() == 0;
        }

        public void printCompilationErrors() {
            for (IProblem error : errors) {
                System.out.println(error + " " + error.getSourceStart());
            }
        }

        public boolean hasError() {
            if (errors == null) {
                return false;
            }
            for (IProblem problem : errors) {
                if (problem.isError()) {
                    return true;
                }
            }
            return false;
        }

        public boolean hasErrorOtherThan(String... ignore) {
            if (errors == null) {
                return false;
            }
            for (int i = 0; i < errors.size(); i++) {
                if (errors.get(i).isError()) {
                    int j = 0;
                    for (; j < ignore.length; j++) {
                        if (errors.get(i).getMessage().contains(ignore[j])) {
                            break;
                        }
                    }
                    if (j == ignore.length) {
                        return true;
                    }
                }
            }
            return false;
        }

        public boolean hasMatchingError(String match) {
            if (errors == null) {
                return false;
            }
            for (IProblem problem : errors) {
                if (problem.isError() && problem.getMessage().contains(match)) {
                    return true;
                }
            }
            return false;
        }
    }

    /** Unique instance */
    private static ICUCompiler inst = new ICUCompiler();

    /** Result of the compilation */
    private ICUCompilerResult result;

    public static ICUCompiler inst() {
        return inst;
    }

    @Override
    public void acceptProblem(IProblem problem) {
        if (problem.isError()) {
            result.errors.add(problem);
        }
    }

    @Override
    public void beginReporting() {
    }

    @Override
    public void endReporting() {
    }

    @Override
    public boolean isActive() {
        return true;
    }

    @SuppressWarnings("deprecation")
    public ICUCompilerResult compile(ICompilationUnit icu) throws JavaModelException {
        result = new ICUCompilerResult();
        icu.becomeWorkingCopy(inst, null);
        icu.discardWorkingCopy();
        return result;
    }
}
