package coco.runtime;

import java.io.File;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import org.junit.runner.JUnitCore;
import org.junit.runner.Result;

import coring.OptParser;
import coring.OptParserStatus;

/**
 * Runner executes all provided test cases and prints/measures predicate
 * coverage.
 * 
 * @author Milos Gligoric (milos.gligoric@gmail.com)
 * @author Rohan Sharma (sharma.rohan1990@gmail.com)
 */
public class CoCoRunner {

    private static CoCoRuntimeOpt options;
    private static CoCoRuntimeListener runtimeListener;

    public static void main(String[] args) {
        try {
            options = new CoCoRuntimeOpt();
            OptParserStatus status = OptParser.helpOrParse(options, args);
            if (status.isSuccessfullyParsed()) {
                CoCoRuntime.inst().setSaveTouchedToFile(false);
                runtimeListener = new CoCoRuntimeListener(options);
                runtimeListener.bcRunStarted();
                runTestClasses();
                runtimeListener.bcRunFinished();
            } else if (status.isError()) {
                System.out.println(status.getExceptionMsg());
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    // PRIVATE

    private static void runTestClasses() throws ClassNotFoundException {
        if (options.getTestClassNames() != null) {
            runTestClasses(options.getTestClassNames());
        }
        if (options.getTestDirectory() != null) {
            runTestClasses(options.getTestDirectory());
        }
    }

    private static void runTestClasses(String testDirectory) throws ClassNotFoundException {
        File dir = new File(testDirectory);
        List<String> testClassNames = new LinkedList<String>();
        LinkedList<File> work = new LinkedList<File>();
        work.add(dir);
        while (!work.isEmpty()) {
            File current = work.removeFirst();
            if (current.isDirectory()) {
                work.addAll(Arrays.asList(current.listFiles()));
            } else if (current.getName().endsWith(".java")) { // it is file
                String testClassName = current.getPath().replaceAll(options.getDirectoryPrefix(), "")
                        .replaceAll("\\.java", "").replaceAll("/", "\\.");
                testClassNames.add(testClassName);
            }
        }
        runTestClasses(testClassNames);
    }

    private static void runTestClasses(List<String> testClassNames) throws ClassNotFoundException {
        for (String testClassName : testClassNames) {
            runTestClass(testClassName);
        }
    }

    private static Result runTestClass(String testClassName) throws ClassNotFoundException {
        JUnitCore junitCore = new JUnitCore();
        junitCore.addListener(runtimeListener);
        return junitCore.run(Class.forName(testClassName));
    }

}
