package coco.runtime;

import java.util.List;

import coring.BooleanOpt;
import coring.IntegerOpt;
import coring.OptHelp;
import coring.StringLst;
import coring.StringOpt;

/**
 * Command line options for {@link CoCoRunner}.
 * 
 * @author Milos Gligoric (milos.gligoric@gmail.com)
 * @author Rohan Sharma (sharma.rohan1990@gmail.com)
 */
class CoCoRuntimeOpt {

    @OptHelp("Prefix that should be removed to form a package name.")
    public final StringOpt directoryPrefix = new StringOpt("directory-prefix");

    @OptHelp("Directory that contains test classes to run.")
    public final StringOpt testDirectory = new StringOpt("test-directory");

    @OptHelp("Comma separated names of test classes to run.")
    public final StringLst testClassNames = new StringLst("test-class-names");

    @OptHelp("Print macros for all values.")
    public final BooleanOpt isUseNewcommand = new BooleanOpt("use-newcommand").s(false);

    @OptHelp("String to prefix macro for newcommand.")
    public final StringOpt newcommandPrefix = new StringOpt("newcommand-prefix").s("");

    @OptHelp("String to suffix macro for newcommand.")
    public final StringOpt newcommandSuffix = new StringOpt("newcommand-suffix").s("");

    @OptHelp("Text used when printing macro for number of states.")
    public final StringOpt statesText = new StringOpt("states-text").s("NumOfStates");

    @OptHelp("Text used when printing macro for execution time.")
    public final StringOpt timeText = new StringOpt("time-text").s("ExecutionTime");

    @OptHelp("Text used when printing macro for number of test classes.")
    public final StringOpt testClassText = new StringOpt("test-class-text").s("NumOfTestClasses");

    @OptHelp("Text used when printing macro for numbrer of tests.")
    public final StringOpt testCaseText = new StringOpt("test-case-text").s("NumOfTestCases");

    @OptHelp("Text used when printing macro for number of failed tests.")
    public final StringOpt failureCaseText = new StringOpt("failure-case-text").s("NumOfFailureCases");

    @OptHelp("Number of predicates.")
    public final IntegerOpt numOfPredicates = new IntegerOpt("num-of-predicates");

    @OptHelp("Name of the file to be use to save coverage.")
    public final StringOpt saveCoveragePerTestFileName = new StringOpt("save-coverage-file-name");

    public String getDirectoryPrefix() {
        return directoryPrefix.g();
    }

    public String getTestDirectory() {
        return testDirectory.g();
    }

    public List<String> getTestClassNames() {
        return testClassNames.g();
    }

    public boolean isUseNewcommand() {
        return isUseNewcommand.g();
    }

    public String getNewcommandPrefix() {
        return newcommandPrefix.g();
    }

    public String getNewcommandSuffix() {
        return newcommandSuffix.g();
    }

    public String getStatesText() {
        return statesText.g();
    }

    public String getNewcommandTimeText() {
        return timeText.g();
    }

    public String getNewcommandTestClassText() {
        return testClassText.g();
    }

    public String getNewcommandTestCaseText() {
        return testCaseText.g();
    }

    public String getNewcommandFailureCaseText() {
        return failureCaseText.g();
    }

    public int getNumOfPredicates() {
        return numOfPredicates.g();
    }

    public String getSaveCoveragePerTestFileName() {
        return saveCoveragePerTestFileName.g();
    }
}
