package example.taco.avltree;

public class AvlTree {

    private static AvlNode doubleWithLeftChild(final AvlNode k3) {
        k3.left = AvlTree.rotateWithRightChild(k3.left);
        return AvlTree.rotateWithLeftChild(k3);
    }

    private static AvlNode doubleWithRightChild(final AvlNode k1) {
        k1.right = AvlTree.rotateWithLeftChild(k1.right);
        return AvlTree.rotateWithRightChild(k1);
    }

    private static int height(final AvlNode t) {
        return t == null ? -1 : t.height;
    }

    private static int max(final int lhs, final int rhs) {
        return lhs > rhs ? lhs : rhs;
    }

    private static AvlNode rotateWithLeftChild(final AvlNode k2) {
        final AvlNode k1 = k2.left;
        k2.left = k1.right;
        k1.right = k2;
        k2.height = AvlTree.max(AvlTree.height(k2.left), AvlTree.height(k2.right)) + 1;
        k1.height = AvlTree.max(AvlTree.height(k1.left), k2.height) + 1;
        return k1;
    }

    private static AvlNode rotateWithRightChild(final AvlNode k1) {
        final AvlNode k2 = k1.right;
        k1.right = k2.left;
        k2.left = k1;
        k1.height = AvlTree.max(AvlTree.height(k1.left), AvlTree.height(k1.right)) + 1;
        k2.height = AvlTree.max(AvlTree.height(k2.right), k1.height) + 1;
        return k2;
    }

    private AvlNode root;

    public AvlTree() {
        this.root = null;
    }

    @Override
    public String toString() {
        return root != null ? root.toString() : "null";
    }

    boolean balanced() {
        return balanced(this.root);
    }

    private boolean balanced(final AvlNode an) {
        if (an == null) {
            return true;
        }

        final int lh = AvlTree.height(an.left);
        final int rh = AvlTree.height(an.right);
        return ((lh == rh) || (lh == rh + 1) || (lh + 1 == rh)) && balanced(an.left) && balanced(an.right);
    }

    private int elementAt(final AvlNode t) {
        return t == null ? -1 : t.element;
    }

    public int find(final int x) {
        return elementAt(find(x, this.root));
    }

    public AvlNode findNode(final int x) {
        return find(x, this.root);
    }

    private AvlNode find(final int x, final AvlNode arg) {
        AvlNode t = arg;
        while (t != null) {
            if (x < t.element) {
                t = t.left;
            } else {
                if (x > t.element) {
                    t = t.right;
                } else {
                    return t;
                }
            }
        }

        return null;
    }

    public int findMax() {
        return elementAt(findMax(this.root));
    }

    public AvlNode fmax() {
        return findMax(this.root);
    }

    private AvlNode findMax(final AvlNode arg) {
        AvlNode t = arg;
        if (t == null) {
            return t;
        }
        while (t.right != null) {
            t = t.right;
        }
        return t;
    }

    public int findMin() {
        return elementAt(findMin(this.root));
    }

    private AvlNode findMin(AvlNode t) {
        if (t == null) {
            return t;
        }
        while (t.left != null) {
            t = t.left;
        }
        return t;
    }

    public void insertElem(final int x) {
        this.root = insert(x, this.root);
    }

    private AvlNode insert(final int x, final AvlNode arg) {
        AvlNode t = arg;
        if (t == null) {
            t = new AvlNode(x, null, null);
        } else {
            if (x < t.element) {
                t.left = insert(x, t.left);
                if (AvlTree.height(t.left) - AvlTree.height(t.right) == 2) {
                    if (x < t.left.element) {
                        t = AvlTree.rotateWithLeftChild(t);
                    } else {
                        t = AvlTree.doubleWithLeftChild(t);
                    }
                }
            } else {
                if (x > t.element) {
                    t.right = insert(x, t.right);
                    if (AvlTree.height(t.right) - AvlTree.height(t.left) == 2) {
                        if (x > t.right.element) {
                            t = AvlTree.rotateWithRightChild(t);
                        } else {
                            t = AvlTree.doubleWithRightChild(t);
                        }
                    }
                }
            }
        }

        t.height = AvlTree.max(AvlTree.height(t.left), AvlTree.height(t.right)) + 1;
        return t;
    }

    @SuppressWarnings("unused")
    private AvlNode insert_0(final int x, final AvlNode arg) {
        AvlNode t = arg;
        if (t == null) {
            t = new AvlNode(x, null, null);
        } else {
            if (x < t.element) {
                t.left = insert_1(x, t.left);
                if (AvlTree.height(t.left) - AvlTree.height(t.right) == 2) {
                    if (x < t.left.element) {
                        t = AvlTree.rotateWithLeftChild(t);
                    } else {
                        t = AvlTree.doubleWithLeftChild(t);
                    }
                }
            } else {
                if (x > t.element) {
                    t.right = insert_1(x, t.right);
                    if (AvlTree.height(t.right) - AvlTree.height(t.left) == 2) {
                        if (x > t.right.element) {
                            t = AvlTree.rotateWithRightChild(t);
                        } else {
                            t = AvlTree.doubleWithRightChild(t);
                        }
                    }
                }
            }
        }

        t.height = AvlTree.max(AvlTree.height(t.left), AvlTree.height(t.right)) + 1;
        return t;
    }

    private AvlNode insert_1(final int x, final AvlNode arg) {
        AvlNode t = arg;
        if (t == null) {
            t = new AvlNode(x, null, null);
        } else {
            if (x < t.element) {
                t.left = insert_2(x, t.left);
                if (AvlTree.height(t.left) - AvlTree.height(t.right) == 2) {
                    if (x < t.left.element) {
                        t = AvlTree.rotateWithLeftChild(t);
                    } else {
                        t = AvlTree.doubleWithLeftChild(t);
                    }
                }
            } else {
                if (x > t.element) {
                    t.right = insert_2(x, t.right);
                    if (AvlTree.height(t.right) - AvlTree.height(t.left) == 2) {
                        if (x > t.right.element) {
                            t = AvlTree.rotateWithRightChild(t);
                        } else {
                            t = AvlTree.doubleWithRightChild(t);
                        }
                    }
                }
            }
        }

        t.height = AvlTree.max(AvlTree.height(t.left), AvlTree.height(t.right)) + 1;
        return t;
    }

    private AvlNode insert_2(final int x, final AvlNode arg) {
        AvlNode t = arg;
        if (t == null) {
            t = new AvlNode(x, null, null);
        } else {
            if (x < t.element) {
                t.left = insert_3(x, t.left);
                if (AvlTree.height(t.left) - AvlTree.height(t.right) == 2) {
                    if (x < t.left.element) {
                        t = AvlTree.rotateWithLeftChild(t);
                    } else {
                        t = AvlTree.doubleWithLeftChild(t);
                    }
                }
            } else {
                if (x > t.element) {
                    t.right = insert_3(x, t.right);
                    if (AvlTree.height(t.right) - AvlTree.height(t.left) == 2) {
                        if (x > t.right.element) {
                            t = AvlTree.rotateWithRightChild(t);
                        } else {
                            t = AvlTree.doubleWithRightChild(t);
                        }
                    }
                }
            }
        }

        t.height = AvlTree.max(AvlTree.height(t.left), AvlTree.height(t.right)) + 1;
        return t;
    }

    private AvlNode insert_3(final int x, final AvlNode arg) {
        AvlNode t = arg;
        if (t == null) {
            t = new AvlNode(x, null, null);
        } else {
            if (x < t.element) {
                t.left = insert_4(x, t.left);
                if (AvlTree.height(t.left) - AvlTree.height(t.right) == 2) {
                    if (x < t.left.element) {
                        t = AvlTree.rotateWithLeftChild(t);
                    } else {
                        t = AvlTree.doubleWithLeftChild(t);
                    }
                }
            } else {
                if (x > t.element) {
                    t.right = insert_4(x, t.right);
                    if (AvlTree.height(t.right) - AvlTree.height(t.left) == 2) {
                        if (x > t.right.element) {
                            t = AvlTree.rotateWithRightChild(t);
                        } else {
                            t = AvlTree.doubleWithRightChild(t);
                        }
                    }
                }
            }
        }

        t.height = AvlTree.max(AvlTree.height(t.left), AvlTree.height(t.right)) + 1;
        return t;
    }

    private AvlNode insert_4(final int x, final AvlNode arg) {
        AvlNode t = arg;
        if (t == null) {
            t = new AvlNode(x, null, null);
        } else {
            if (x < t.element) {
                throw new RuntimeException();
            } else {
                if (x > t.element) {
                    throw new RuntimeException();
                }
            }
        }

        t.height = AvlTree.max(AvlTree.height(t.left), AvlTree.height(t.right)) + 1;
        return t;
    }

    public boolean isEmpty() {
        return this.root == null;
    }

    public void makeEmpty() {
        this.root = null;
    }

    boolean maxElement(final int max) {
        return maxElement(max, this.root);
    }

    private boolean maxElement(final int max, final AvlNode t) {
        if (t == null) {
            return true;
        }
        if (max < t.element) {
            return false;
        }
        return maxElement(max, t.left) && maxElement(max, t.right);
    }

    boolean minElement(final int min) {
        return minElement(min, this.root);
    }

    private boolean minElement(final int min, final AvlNode t) {
        if (t == null) {
            return true;
        }
        if (min > t.element) {
            return false;
        }
        return minElement(min, t.left) && minElement(min, t.right);
    }

    public void remove(final int x) {
    }

    boolean wellFormed() {
        return wellFormed(this.root);
    }

    private int Math_max(int l, int r) {
        return (l > r ? l : r);
    }

    private boolean wellFormed(final AvlNode an) {
        if (an == null) {
            return true;
        }
        if (AvlTree.height(an) != Math_max(AvlTree.height(an.left), AvlTree.height(an.right)) + 1) {
            return false;
        }
        return wellFormed(an.left) && wellFormed(an.right);
    }

}
