package example.taco.linkedlist;

public class LinkedList {

    public transient LinkedListNode header;

    protected transient int size;

    protected transient int modCount;

    public LinkedList() {
        init();
        this.size = 0;
    }

    @Override
    public String toString() {
        if (header == null) {
            return "null";
        } else {
            LinkedListNode c = header;
            String s = c.toString();
            while (c.next != header) {
                c = c.next;
                s += ":" + c.toString();
            }
            return s;
        }
    }

    protected void init() {
        header = createHeaderNode();
    }

    public int size() {
        return size;
    }

    public boolean isEmpty() {
        return (size() == 0);
    }

    public Object get(int index) {
        LinkedListNode node = getNode(index, false);
        return node.getValue();
    }

    public int indexOf(Object value) {
        int i = 0;
        for (LinkedListNode node = header.next; node != header; node = node.next) {
            if (isEqualValue(node.getValue(), value)) {
                return i;
            } else {
            }
            i++;
        }
        return -1;
    }

    public int lastIndexOf(Object value) {
        int i = size - 1;
        for (LinkedListNode node = header.previous; node != header; node = node.previous) {
            if (isEqualValue(node.getValue(), value)) {
                return i;
            }
            i--;
        }
        return -1;
    }

    public boolean contains(Object arg) {
        return indexOf(arg) != -1;
    }

    public boolean add(Object value) {
        addLast(value);
        return true;
    }

    public void add(int index, Object value) {
        LinkedListNode node = getNode(index, true);
        addNodeBefore(node, value);
    }

    public Object removeIndex(int index) {
        LinkedListNode node = getNode(index, false);
        Object oldValue = node.getValue();
        removeNode(node);
        return oldValue;
    }

    public boolean remove(Object value) {
        for (LinkedListNode node = header.next; node != header; node = node.next) {
            if (isEqualValue(node.getValue(), value)) {
                removeNode(node);
                return true;
            }
        }
        return false;
    }

    public Object set(int index, Object value) {
        LinkedListNode node = getNode(index, false);
        Object oldValue = node.getValue();
        updateNode(node, value);
        return oldValue;
    }

    public void clear() {
        removeAllNodes();
    }

    public Object getFirst() {
        LinkedListNode node = header.next;
        if (node == header) {
            throw new RuntimeException();
        }
        return node.getValue();
    }

    public Object getLast() {
        LinkedListNode node = header.previous;
        if (node == header) {
            throw new RuntimeException();
        }
        return node.getValue();
    }

    public boolean addFirst(Object o) {
        addNodeAfter(header, o);
        return true;
    }

    public boolean addLast(Object o) {
        addNodeBefore(header, o);
        return true;
    }

    public Object removeFirst() {
        LinkedListNode node = header.next;
        if (node == header) {
            throw new RuntimeException();
        }
        Object oldValue = node.getValue();
        removeNode(node);
        return oldValue;
    }

    public Object removeLast() {
        LinkedListNode node = header.previous;
        if (node == header) {
            throw new RuntimeException();
        }
        Object oldValue = node.getValue();
        removeNode(node);
        return oldValue;
    }

    protected boolean isEqualValue(Object value1, Object value2) {
        return (value1 == value2 || (value1 == null ? false : value1.equals(value2)));
    }

    protected void updateNode(LinkedListNode node, Object value) {
        node.setValue(value);
    }

    protected LinkedListNode createHeaderNode() {
        return new LinkedListNode();
    }

    protected LinkedListNode createNode(Object value) {
        return new LinkedListNode(value);
    }

    protected void addNodeBefore(LinkedListNode node, Object value) {
        LinkedListNode newNode = createNode(value);
        addNode(newNode, node);
    }

    protected void addNodeAfter(LinkedListNode node, Object value) {
        LinkedListNode newNode = createNode(value);
        addNode(newNode, node.next);
    }

    protected void addNode(LinkedListNode nodeToInsert, LinkedListNode insertBeforeNode) {
        nodeToInsert.next = insertBeforeNode;
        nodeToInsert.previous = insertBeforeNode.previous;
        insertBeforeNode.previous.next = nodeToInsert;
        insertBeforeNode.previous = nodeToInsert;
        size++;
        modCount++;
    }

    protected void removeNode(LinkedListNode node) {
        node.previous.next = node.next;
        node.next.previous = node.previous;
        size--;
        modCount++;
    }

    protected void removeAllNodes() {
        header.next = header;
        header.previous = header;
        size = 0;
        modCount++;
    }

    protected LinkedListNode getNode(int index, boolean endMarkerAllowed) throws IndexOutOfBoundsException {
        if (index < 0) {
            throw new IndexOutOfBoundsException("Couldn't get the node: " + "index (" + index + ") less than zero.");
        }
        if (!endMarkerAllowed && index == size) {
            throw new IndexOutOfBoundsException("Couldn't get the node: " + "index (" + index
                    + ") is the size of the list.");
        }
        if (index > size) {
            throw new IndexOutOfBoundsException("Couldn't get the node: " + "index (" + index
                    + ") greater than the size of the " + "list (" + size + ").");
        }
        LinkedListNode node;
        if (index < (size / 2)) {
            node = header.next;
            for (int currentIndex = 0; currentIndex < index; currentIndex++) {
                node = node.next;
            }
        } else {
            node = header;
            for (int currentIndex = size; currentIndex > index; currentIndex--) {
                node = node.previous;
            }
        }
        return node;
    }

}
