#!/bin/bash

plugin_workspace=workspace/
plugin_main=coco.CoCoInstrumentation

eclipse_dir=eclipse
if [ ! -d ${eclipse_dir} ]; then
    echo "0. Downloading eclipse ..."
    platform=`uname -m`
    if [ `uname -s` == "Darwin" ]; then
        # Assume x64
        wget http://archive.eclipse.org/technology/epp/downloads/release/helios/SR2/eclipse-jee-helios-SR2-macosx-cocoa-x86_64.tar.gz
    elif [ "${platform}" = "x86_64" ]; then
        wget http://archive.eclipse.org/technology/epp/downloads/release/helios/SR2/eclipse-jee-helios-SR2-linux-gtk-x86_64.tar.gz
    else
        wget http://archive.eclipse.org/technology/epp/downloads/release/helios/SR2/eclipse-jee-helios-SR2-linux-gtk.tar.gz
    fi
    tar xfz eclipse*.gz
    if [ $? -ne 0 ]; then exit 1; fi
    cp plugins/*.jar ${eclipse_dir}/plugins/
fi
ECLIPSE_HOME=${eclipse_dir}
ECLIPSE_LAUNCHER=`find ${ECLIPSE_HOME} -type f -name "*equinox.launcher_*.jar"`

bc_flag="bc.coco.flag"
sc_flag="sc.coco.flag"
path_flag="path.coco.flag"
pctbb_flag="pctbb.coco.flag"
pctst_flag="pctst.coco.flag"
pctms_flag="pctms.coco.flag"

function option_help_fun() {
    echo "    --$1 project_name project_location test_classes"
    echo "        Instrument given project for measuring $2 and run test classes"
}

function help_fun() {
    echo "${0} options [arguments]"
    echo
    option_help_fun "pctbb" "PCT(BB)"
    echo
    option_help_fun "pctst" "PCT(ST)"
    echo
    echo "    --pctms"
    echo "        PCT(MS) is manual instrumentation available at: http://mir.cs.illinois.edu/coverage/"
    echo
    option_help_fun "bc" "BC"
    echo
    option_help_fun "sc" "SC"
    echo
    option_help_fun "path" "Path coverage"
    echo
    echo "    --help"
    echo "        Prints this message"
}

function instrument_fun() {
    compilation_units=$(find ${in_project_name}/src -name "*.java" | sed 's/'${in_project_name}'\/src\///g' | sed 's/\.java//g' | sed 's/\//\./g' | tr '\n' ',')

    echo "Instrumenting code for measuring ${task_text}"
    java -cp ${ECLIPSE_LAUNCHER} org.eclipse.core.launcher.Main -data ${plugin_workspace} \
        -application ${plugin_main} \
        --compilation-units ${compilation_units} \
        --project-name ${in_project_name} \
        --project-location ${in_project_location} \
        --new-project-name ${out_project_name} \
        --rename-variables \
        --extract-predicates \
        --insert-basic-blocks \
        --instantiate-predicates \
        "$@"
}

function run_fun() {
    echo "Running test cases"
    ( cd workspace/${out_project_name}; ant clean; ant )
    java -classpath workspace/${out_project_name}/lib/*:workspace/${out_project_name}/bin \
        coco.runtime.CoCoRunner \
        --test-class-names ${test_classes}
}

function instrument_and_run_fun() {
    # Script arguments
    task_text=${1}; shift
    out_project_name=${1}; shift
    # User arguments
    in_project_name=${1}; shift
    in_project_location=${1}; shift
    test_classes=${1}; shift

    instrument_fun "$@"
    run_fun
}

rm -rf *.coco.flag
rm -rf ${plugin_workspace}.metadata
#mkdir -p ${plugin_workspace}

if [ $# -lt 3 ]; then
    help_fun "$@"
elif [ "$1" = "--pctbb" ]; then shift
    rm -rf ${plugin_workspace}pctbb
    touch ${pctbb_flag}
    instrument_and_run_fun "PCT(BB)" "pctbb" "$@"
elif [ "$1" = "--pctst" ]; then shift
    rm -rf ${plugin_workspace}pctst
    touch ${pctst_flag}
    instrument_and_run_fun "PCT(ST)" "pctst" "$@" "--instantiate-statement"
elif [ "$1" = "--pctms" ]; then shift
    echo "PCT(MS) is manual instrumentation available at: http://mir.cs.illinois.edu/coverage/"
elif [ "$1" = "--bc" ]; then shift
    rm -rf ${plugin_workspace}bc
    touch ${bc_flag}
    instrument_and_run_fun "BC" "bc" "$@" "--branch-coverage"
elif [ "$1" = "--sc" ]; then shift
    rm -rf ${plugin_workspace}sc
    touch ${sc_flag}
    instrument_and_run_fun "SC" "sc" "$@" "--statement-coverage"
elif [ "$1" = "--path" ]; then shift
    rm -rf ${plugin_workspace}path
    touch ${path_flag}
    instrument_and_run_fun "Path" "path" "$@"
else
    help_fun "$@"
fi
rm -rf *.coco.flag
